import re
import json

from rest_framework.parsers import JSONParser, ParseError, six
from django.conf import settings
from util import get_instance_wrapper


__authors__ = ["Vitaly Babiy", "Joakim Soderlund"]
__copyright__ = "Copyright (c) 2013, Vitaly Babiy"
__license__ = "BSD"

FIRST_CAPS_RE = re.compile('(.)([A-Z][a-z]+)')
EVERY_CAPS_RE = re.compile('([a-z0-9])([A-Z])')


def camel_to_underscore(name):
	sub = FIRST_CAPS_RE.sub(r'\1_\2', name)
	return EVERY_CAPS_RE.sub(r'\1_\2', sub).lower()


def decamelize(data):
	if isinstance(data, dict):
		new_dict = {}
		for key, value in data.items():
			new_key = camel_to_underscore(key)
			new_dict[new_key] = decamelize(value)
		return new_dict

	if isinstance(data, (list, tuple)):
		for i in range(len(data)):
			data[i] = decamelize(data[i])
		return data

	return data


class EmberParser(JSONParser):
	"""
	Parser specifically designed for Ember.
	"""

	media_type = 'application/ember'
	format = 'ember'

	def parse(self, stream, media_type=None, parser_context=None):
		parser_context = parser_context or {}
		encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

		try:
			data = json.loads(stream.read().decode(encoding))
			data = decamelize(data)

			wrapper = get_instance_wrapper(parser_context)

			if wrapper in data:
				data = data[wrapper]
			else:
				raise ParseError("Expected key \"{}\" in data.".format(wrapper))

			return data

		except ValueError as exc:
			raise ParseError('JSON parse error: %s' % six.text_type(exc))
